This patch allows you to override the primary VID/DID, subsystem VID/DID
PCI device class, and combined PCI revision and prog-if.

Using this is generally a really bad idea unless you know exactly what you're
doing; an example of a device that this is usable with is the HighPoint 1144CM
which is actually 4 ASmedia 1042 XHCI controllers behind a PCI-e switch, reporting
a "fake" PCI type as a RAID controller. Also might be useful for passing through
NVidia cards without hard modifying them.

diff -Nur v/qemu-2.2.0/hw/misc/vfio.c qemu-2.2.0/hw/misc/vfio.c
--- v/qemu-2.2.0/hw/misc/vfio.c	2014-12-09 14:45:41.000000000 +0000
+++ qemu-2.2.0/hw/misc/vfio.c	2015-02-27 23:46:23.592614925 +0000
@@ -218,6 +218,12 @@
 #define VFIO_FEATURE_ENABLE_VGA_BIT 0
 #define VFIO_FEATURE_ENABLE_VGA (1 << VFIO_FEATURE_ENABLE_VGA_BIT)
     int32_t bootindex;
+    uint16_t virtual_vendor_id;
+    uint16_t virtual_device_id;
+    uint16_t virtual_ss_vendor_id;
+    uint16_t virtual_ss_device_id;
+    uint16_t virtual_class;
+    uint16_t virtual_rev_and_prog_if;
     uint8_t pm_cap;
     bool reset_works;
     bool has_vga;
@@ -4262,6 +4268,32 @@
     memset(&vdev->pdev.config[PCI_BASE_ADDRESS_0], 0, 24);
     memset(&vdev->pdev.config[PCI_ROM_ADDRESS], 0, 4);
 
+    if (vdev->virtual_vendor_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_VENDOR_ID,
+                               cpu_to_le16(vdev->virtual_vendor_id), 0xFFFF);
+    }
+    if (vdev->virtual_device_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_DEVICE_ID,
+                               cpu_to_le16(vdev->virtual_device_id), 0xFFFF);
+    }
+    if (vdev->virtual_ss_vendor_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_SUBSYSTEM_VENDOR_ID,
+                               cpu_to_le16(vdev->virtual_ss_vendor_id), 0xFFFF);
+    }
+    if (vdev->virtual_ss_device_id != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_SUBSYSTEM_ID,
+                               cpu_to_le16(vdev->virtual_ss_device_id), 0xFFFF);
+    }
+    if (vdev->virtual_class != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_CLASS_DEVICE,
+                               cpu_to_le16(vdev->virtual_class), 0xFFFF);
+    }
+    if (vdev->virtual_rev_and_prog_if != 0xFFFF) {
+        vfio_add_emulated_word(vdev, PCI_CLASS_REVISION,
+                               cpu_to_le16(vdev->virtual_rev_and_prog_if), 0xFFFF);
+    }
+
+
     vfio_pci_size_rom(vdev);
 
     ret = vfio_early_setup_msix(vdev);
@@ -4381,6 +4413,12 @@
                        intx.mmap_timeout, 1100),
     DEFINE_PROP_BIT("x-vga", VFIODevice, features,
                     VFIO_FEATURE_ENABLE_VGA_BIT, false),
+    DEFINE_PROP_UINT16("x-vid", VFIODevice, virtual_vendor_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-did", VFIODevice, virtual_device_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-ss-vid", VFIODevice, virtual_ss_vendor_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-ss-did", VFIODevice, virtual_ss_device_id, 0xFFFF),
+    DEFINE_PROP_UINT16("x-class", VFIODevice, virtual_class, 0xFFFF),
+    DEFINE_PROP_UINT16("x-rev-and-progif", VFIODevice, virtual_rev_and_prog_if, 0xFFFF),
     /*
      * TODO - support passed fds... is this necessary?
      * DEFINE_PROP_STRING("vfiofd", VFIODevice, vfiofd_name),
